"""
Mimicry
------------

The Mimicry app provides functionality for simulating user sessions
and actions, allowing site administrators to view the website as if
they are a specific user, aiding in debugging and user experience enhancements.

This module sets the default application configuration class.
"""

# Point to the default AppConfig class for the 'mimicry' app.
default_app_config = 'mimicry.apps.MimicryConfig'
