# Generated by Django 4.2.4 on 2023-08-11 22:17

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SimulatedUserSession',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('real_session_key', models.CharField(max_length=40)),
                ('simulated_session_key', models.CharField(max_length=40, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('closed_at', models.DateTimeField(blank=True, null=True)),
                ('hide_bar', models.BooleanField(default=False)),
                ('real_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='real_user_sessions', to=settings.AUTH_USER_MODEL)),
                ('simulated_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='simulated_user_sessions', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SimulatedUserAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.TextField()),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('simulated_user_session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='simulated_session_actions', to='mimicry.simulatedusersession')),
            ],
        ),
    ]
